<?php

namespace Drupal\Tests\plugin_form_element\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Functional tests for the Plugins form element.
 *
 * @group plugin_form_element
 */
class PluginsFormElementTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'plugin_form_element',
    'plugin_form_element_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the rendering of the Plugins form element.
   */
  public function testRenderPluginsFormElement() {
    $this->drupalLogin($this->drupalCreateUser(['administer site configuration']));

    $this->drupalGet('/plugin-form-element-test/plugins');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Configure plugins');
    $this->assertSession()->elementExists('css', '.form-item--plugins-uuid-1-configuration');
  }

  /**
   * Tests AJAX add and remove operations for Plugins.
   */
  public function testAjaxAddRemovePlugins() {
    $this->drupalLogin($this->drupalCreateUser(['administer site configuration']));

    $this->drupalGet('/plugin-form-element-test/plugins');
    $this->assertSession()->statusCodeEquals(200);

    // Trigger the AJAX 'add plugin' operation.
    $add_button = $this->getSession()->getPage()->findButton('Add Current ThemeX');
    $this->assertNotNull($add_button, 'Add button is present.');
    $add_button->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertSession()->elementExists('css', '.plugins-plugin-configuration--theme');

    // Trigger the AJAX 'remove plugin' operation.
    $this->getSession()->getPage()->pressButton('RemoveX');

    $remove_button = $this->getSession()->getPage()->findButton('Remove');
    $this->assertNotNull($add_button, 'Remove button is present.');
    $remove_button->click();
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertSession()->elementExists('css', '.plugins-plugin-configuration--theme');
  }

  /**
   * Tests form submission with Plugins form element.
   */
  public function testFormSubmission() {
    $this->drupalLogin($this->drupalCreateUser(['administer site configuration']));

    $this->drupalGet('/plugin-form-element-test/plugins');
    $this->assertSession()->statusCodeEquals(200);

    // Fill in plugin configurations and submit the form.
    $this->getSession()->getPage()->fillField('edit-plugins[uuid_1][configuration]', 'Some value');
    $this->getSession()->getPage()->pressButton('Save');

    // Ensure the form is submitted and values are processed.
    $this->assertSession()->pageTextContains('Plugins configuration saved');
  }

}
