<?php

namespace Drupal\Tests\plugin_form_element\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Core\Form\FormState;
use Drupal\plugin_form_element\Element\Plugin;

/**
 * Kernel test for the Plugin form element.
 *
 * @group plugin_form_element
 */
class PluginFormElementTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'plugin_form_element',
    'system',
    'user',
    'language',
  ];

  /**
   * Tests the form element initialization.
   */
  public function testFormElementInitialization() {
    $form_state = new FormState();
    $element = [
      '#type' => 'plugin',
      '#plugin_manager' => 'plugin.manager.condition',
      '#default_value' => [
        'id' => 'language',
        'configuration' => ['langcodes' => ['en' => 'en']],
      ],
    ];

    $complete_form = [];
    $processed_element = Plugin::processPluginElement($element, $form_state, $complete_form);
    $this->assertArrayHasKey('#plugin', $processed_element);
    $this->assertEquals('language', $processed_element['#plugin_id']);
  }

  /**
   * Tests the form submission.
   */
  public function testFormElementSubmission() {
    $complete_form = [];
    $form_state = new FormState();
    $form_state->setCompleteForm($complete_form);
    $element = [
      '#type' => 'plugin',
      '#plugin_manager' => 'plugin.manager.condition',
      '#default_value' => [
        'id' => 'language',
        'configuration' => ['langcodes' => ['en' => 'en']],
      ],
    ];

    $processed_element = Plugin::processPluginElement($element, $form_state, $complete_form);
    Plugin::validatePluginForm($processed_element, $form_state, $complete_form);
    $submitted_value = Plugin::valueCallback(
      $processed_element,
      ['langcodes' => ['en' => 'en', 'de' => 'de']],
      $form_state
    );

    $this->assertEquals('language', $submitted_value['id']);
    $this->assertEquals(['en' => 'en', 'de' => 'de'], $submitted_value['langcodes']);
  }

}
